/*
 * Copyright (c) 2006 Atmark Techno, Inc.  All Rights Reserved.
 *  Masahiro NAKAI <nakai@atmark-techno.com>
 *
 *
 */

#include <target/buffer.h>
#include <target/io.h>
#include <target/herrno.h>
#include <target/scan.h>
#include <target/flash.h>
#include <target/memzero.h>

#define DLBUFSIZE (8*1024)
u8 dlbuf [DLBUFSIZE];
u32 dlbufsize = DLBUFSIZE;

flash_ops fops;


int flash_initialize(int type)
{
	memzero(&fops, sizeof(flash_ops));

	switch(type){
	case FLASH_TYPE_AMD:
		fops.initialize   = flash_amd_initialize;
		fops.program      = flash_amd_program;
		fops.erase        = flash_amd_erase;
		fops.read_byte    = flash_amd_read_byte;
		fops.copy_to_dram = flash_amd_copy_to_dram;
		fops.get_id       = flash_amd_get_id;
		fops.get_size     = flash_amd_get_size;
		break;
	case FLASH_TYPE_INTEL:
		fops.initialize   = flash_intel_initialize;
		fops.program      = flash_intel_program;
		fops.erase        = flash_intel_erase;
		fops.read_byte    = flash_intel_read_byte;
		fops.copy_to_dram = flash_intel_copy_to_dram;
		fops.get_id       = flash_intel_get_id;
		fops.get_size     = flash_intel_get_size;
		break;
	case FLASH_TYPE_SPI:
		fops.initialize   = flash_spi_initialize;
		fops.program      = flash_spi_program;
		fops.erase        = flash_spi_erase;
		fops.read_byte    = flash_spi_read_byte;
		fops.copy_to_dram = flash_spi_copy_to_dram;
		fops.get_id       = flash_spi_get_id;
		fops.get_size     = flash_spi_get_size;
		break;
	default:
		break;
	}
	
	if(fops.initialize) fops.initialize();
	
	return 0;
}

int flash_program(const u32 from, const u32 to, const u32 size)
{
	int ret = -1;

	if(fops.program)
		ret = fops.program((u32)from, to, size);
	
	return ret;
}

static int flash_program_cmdfunc(int argc, char *argv[])
{
	addr_t addr;
	size_t size;

	if (argc != 3)
		return -H_EUSAGE;
	if (scan(*++argv, &addr)) return -H_EADDR;
	if (scan(*++argv, &size)) return -H_EADDR;
	if (addr & UNALIGNED_MASK) return -H_EALIGN;
	if (size & UNALIGNED_MASK) return -H_EALIGN;

	return flash_program((u32)dlbuf, addr, size);
}

const command_t flash_program_command = { "program", "<addr> <size>",
					  "program flash from download buffer",
					  &flash_program_cmdfunc };

int flash_erase(const u32 erase_addr)
{
	int ret = -1;

	if(fops.erase)
		ret = fops.erase(erase_addr);
	
	return ret;
}

static int flash_erase_cmdfunc(int argc, char *argv[])
{
	addr_t addr;

	if (argc != 2)
		return -H_EUSAGE;
	if (scan(*++argv, &addr)) return -H_EADDR;
	addr &= ~(addr_t)UNALIGNED_MASK;

#if defined(ARMADILLOJ_CTYPE)
	//Erase protect
	if(PROTECT_START <= addr && addr <= PROTECT_END){
		return -H_EADDR;
	}
#endif

	return flash_erase(addr);
}

const command_t flash_erase_command = { "erase", "<addr>",
					"erase flash block",
					&flash_erase_cmdfunc };

u32 flash_copy_to_dram(const u32 from, const u32 to, const u32 size)
{
	u32 ret = -1;
  
	if(fops.copy_to_dram)
		ret = fops.copy_to_dram(from, to, size);
	
	return ret;
}

u32 flash_get_id(const u32 base_addr)
{
	u32 ret = -1;
  
	if(fops.get_id)
		ret = fops.get_id(base_addr);
	
	return ret;
}

int flash_get_size(const u32 base_addr)
{
	int ret = -1;
  
	if(fops.get_size)
		ret = fops.get_size(base_addr);
	
	return ret;
}
